/*
 * $Header: /home/cvs/jakarta-tomcat/proposals/catalina/src/share/org/apache/tomcat/core/StandardEngine.java,v 1.3 2000/01/29 07:46:33 craigmcc Exp $
 * $Revision: 1.3 $
 * $Date: 2000/01/29 07:46:33 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 


package org.apache.tomcat.core;


import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;
import org.apache.tomcat.Container;
import org.apache.tomcat.Engine;
import org.apache.tomcat.Host;
import org.apache.tomcat.Request;
import org.apache.tomcat.Response;


/**
 * Standard implementation of the <b>Engine</b> interface.  Each
 * child container must be a Host implementation to process the specific
 * fully qualified host name of that virtual host.
 *
 * @author Craig R. McClanahan
 * @version $Revision: 1.3 $ $Date: 2000/01/29 07:46:33 $
 */

public final class StandardEngine
    extends ContainerBase
    implements Engine {


    // ----------------------------------------------------------- Constructors


    /**
     * Create a new StandardEngine component with the default basic Valve.
     */
    public StandardEngine() {

	super();
	setBasic(new StandardEngineValve());

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * Host name to use when no server host is specified in the request.
     */
    private String defaultHost = null;


    /**
     * The descriptive information string for this implementation.
     */
    private static final String info =
	"org.apache.tomcat.core.StandardEngine/1.0";


    /**
     * Host name to use when an unknown host name is specified.
     */
    private String unknownHost = null;


    // ------------------------------------------------------------- Properties


    /**
     * Return the default host.
     */
    public String getDefaultHost() {

	return (defaultHost);

    }


    /**
     * Set the default host.
     *
     * @param host The new default host
     */
    public void setDefaultHost(String host) {

	String oldDefaultHost = this.defaultHost;
	this.defaultHost = host;
	support.firePropertyChange("defaultHost", oldDefaultHost,
				   this.defaultHost);

    }


    /**
     * Return the unknown host.
     */
    public String getUnknownHost() {

	return (unknownHost);

    }


    /**
     * Set the unknown host.
     *
     * @param host The new unknown host
     */
    public void setUnknownHost(String host) {

	String oldUnknownHost = this.unknownHost;
	this.unknownHost = host;
	support.firePropertyChange("unknownHost", oldUnknownHost,
				   this.unknownHost);

    }


    // --------------------------------------------------------- Public Methods


    /**
     * Add a child Container, only if the proposed child is an implementation
     * of Host.
     *
     * @param child Child container to be added
     */
    public void addChild(Container child) {

	if (!(child instanceof Host))
	    throw new IllegalArgumentException
		(sm.getString("standardEngine.notHost"));
	super.addChild(child);

    }


    /**
     * Return descriptive information about this Container implementation and
     * the corresponding version number, in the format
     * <code>&lt;description&gt;/&lt;version&gt;</code>.
     */
    public String getInfo() {

	return (info);

    }


    /**
     * Return the Host that should be used to process this request,
     * based on the server name that was included.  If no such Host
     * can be identified, return <code>null</code> instead.
     *
     * @param request Request being processed
     */
    public Host map(Request request) {

	// Extract the requested server name
	String server = request.getRequest().getServerName();
	if (server == null)
	    server = defaultHost;
	if (server == null)
	    return (null);

	// Find the matching child Host
	Host host = (Host) findChild(server);
	if ((host == null) && (unknownHost != null))
	    host = (Host) findChild(unknownHost);
	return (host);

    }


    /**
     * Disallow any attempt to set a parent for this Container, since an
     * Engine is supposed to be at the top of the Container hierarchy.
     *
     * @param container Proposed parent Container
     */
    public void setParent(Container container) {

	throw new IllegalArgumentException
	    (sm.getString("standardEngine.notParent"));

    }


}
